// $Id: ServiceTicketValidator.java,v 1.14 2007-11-05 17:14:06+09 naito Exp $ 
//*****************************************************************
// This program is based Yale CAS (cas client for JAVA 2.1.0) 
// This program is CAS^2 client for JAVA (nagoya-u.ac.jp)
//*****************************************************************

package jp.ac.nagoya_u.cas.client;

import java.io.IOException;
import java.io.StringReader;
import java.util.*;
import java.net.URL;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import jp.ac.nagoya_u.cas.client.util.GetURL;
import jp.ac.nagoya_u.cas.client.util.AttributeRewrite;

/**
 * Validates STs and optionally retrieves PGT IOUs.
 * Designed with a bean-like interface for simplicity and generality.
 *
 * @version 1.2
 * @author Based on Yale CAS client for JAVA 2.1.0.
 */
public class ServiceTicketValidator {
    //*********************************************************************
    // Private state

    private String casValidateUrl, proxyCallbackUrl, st, service, pgtIou,
	user, mail, fullName, errorCode, errorMessage, entireResponse ;
    private InputSource inputSource;
    private Map result = new Hashtable() ;
    private boolean renew = false;
    private boolean attemptedAuthentication;
    private boolean successfulAuthentication;
    private boolean useSSL = true ;

    /** Log instance for logging events, info, warnings, errors, etc. */
    private final Log log = LogFactory.getLog(this.getClass());

    /**
     * 
     */
    public void setUseSSL(boolean useSSL) {
	this.useSSL = useSSL ;
    }

    /**
     * Sets the CAS validation URL to use when validating tickets and
     * retrieving PGT IOUs.
     */
    public void setCasValidateUrl(String x) {
	this.casValidateUrl = x;
    }

    /**
     * Gets the CAS validation URL to use when validating tickets and
     * retrieving PGT IOUs.
     */
    public String getCasValidateUrl() {
	return this.casValidateUrl;
    }

    /**
     * Sets the callback URL, owned logically by the calling service, to
     * receive the PGTid/PGTiou mapping.
     */
    public void setProxyCallbackUrl(String x) {
	this.proxyCallbackUrl = x;
    }

    /**
     * Sets the "renew" flag on authentication.  When set to "true",
     * authentication will only succeed if this was an initial login
     * (forced by the "renew" flag being set on login).
     */
    public void setRenew(boolean b) {
	this.renew = b;
    }

    /**
     * Sets the InputSource filename for test XML code
     */
    public void setInputSource(String s) {
	this.inputSource = new InputSource(s) ;
    }

    /**
     * Gets the callback URL, owned logically by the calling service, to
     * receive the PGTid/PGTiou mapping.
     */
    public String getProxyCallbackUrl() {
	return this.proxyCallbackUrl;
    }

    /**
     * Sets the ST to validate.
     */
    public void setServiceTicket(String x) {
	this.st = x;
    }

    /**
     * Sets the service to use when validating.
     */
    public void setService(String x) {
	this.service = x;
    }

    /**
     * Returns the strongly authenticated username.
     */
    public String getUser() {
	return this.user;
    }

    /**
     * Returns the authenticated user's Mail address
     */
    public String getMail() {
	return this.mail;
    }

    /**
     * Returns the authenticated user's Full Name in Japaneses (UTF-8)
     */
    public String getFullName() {
	return this.fullName;
    }

    /**
     * Returns the authenticated user's Attributes hashtable
     */
    public Map getResult() {
	return this.result ;
    }

    /**
     * Returns the PGT IOU returned by CAS.
     */
    public String getPgtIou() {
	return this.pgtIou;
    }

    /**
     * Returns <tt>true</tt> if the most recent authentication attempted
     * succeeded, <tt>false</tt> otherwise.
     */
    public boolean isAuthenticationSuccesful() {
	return this.successfulAuthentication;
    }

    /**
     * Returns an error message if CAS authentication failed.
     */
    public String getErrorMessage() {
	return this.errorMessage;
    }

    /**
     * Returns CAS's error code if authentication failed.
     */
    public String getErrorCode() {
	return this.errorCode;
    }

    /**
     * Retrieves CAS's entire response, if authentication was succsesful.
     */
    public String getResponse()  {
	return this.entireResponse;
    }

    //*********************************************************************
    // For testing XML file

    public void validateFile() throws Exception{
	InputSource response = new InputSource("cas.xml") ;
	if (response != null) {
	    XMLReader r = SAXParserFactory.newInstance().newSAXParser().getXMLReader();
	    r.setFeature("http://xml.org/sax/features/namespaces", false);
	    r.setContentHandler(newHandler());
	    r.parse(response) ;
	    System.out.println(result.toString()) ;
	}
    }

    //*********************************************************************
    // Actuator

    public void validate() throws IOException, SAXException, ParserConfigurationException {
	if (casValidateUrl == null || st == null)
	    throw new IllegalStateException("must set validation URL and ticket");
	clear();
	attemptedAuthentication = true;
	StringBuffer sb = new StringBuffer();
	sb.append(casValidateUrl);
	if (casValidateUrl.indexOf('?') == -1) sb.append('?');
	else sb.append('&');
	sb.append("service=" + service + "&ticket=" + st);
	if (proxyCallbackUrl != null) sb.append("&pgtUrl=" + proxyCallbackUrl);
	if (renew) sb.append("&renew=true");
	String url = sb.toString();

	log.debug("CAS Validate to : " + url) ;

	if (this.useSSL) {
            URL u = new URL(url);
	    if (!u.getProtocol().equals("https")){
              	log.error("retrieve(" + url + ") on an illegal URL since protocol was not https.");
		throw new IOException("only 'https' URLs are valid for this method");
	    }
	}
	String response = GetURL.retrieve(url) ;
	this.entireResponse = response;

	log.debug("RESULT = " + response) ;

	// parse the response and set appropriate properties
	if (response != null) {
	    XMLReader r = SAXParserFactory.newInstance().newSAXParser().getXMLReader();
	    r.setFeature("http://xml.org/sax/features/namespaces", false);
	    r.setContentHandler(newHandler());
	    r.parse(new InputSource(new StringReader(response)));
	}
    }


    //*********************************************************************
    // Response parser

    protected DefaultHandler newHandler() {
	return new Handler();
    }

    protected class Handler extends DefaultHandler {
	protected static final String AUTHENTICATION_SUCCESS = 
	    "cas:authenticationSuccess";
	protected static final String AUTHENTICATION_FAILURE =
	    "cas:authenticationFailure";
	protected static final String PROXY_GRANTING_TICKET =
	    "cas:proxyGrantingTicket";
	protected static final String USER = "cas:user";
	protected static final String USER_NU = "cas:uid";
	protected static final String MAIL = "cas:registMailAddress";
	protected static final String FULLNAME = "cas:fullName";
	protected static final String CAS_TAG = "cas:";
	protected static final String CAS_HEADER = "cas:serviceresponse";

	//**********************************************
	// Parsing state

	protected StringBuffer currentText = new StringBuffer();
	protected boolean authenticationSuccess = false;
	protected boolean authenticationFailure = false;
	protected String netid, pgtIou, errorCode, errorMessage;
    
	private String string ;

	//**********************************************
	// Parsing logic

	public void startElement(String ns, String ln, String qn, Attributes a) {
	    // clear the buffer
	    currentText = new StringBuffer();

	    // check outer elements
	    if (qn.equals(AUTHENTICATION_SUCCESS)) {
		authenticationSuccess = true;
	    } else if (qn.equals(AUTHENTICATION_FAILURE)) {
		authenticationFailure = true;
		errorCode = a.getValue("code");
		if (errorCode != null)
		    errorCode = errorCode.trim();
	    }
	}

	public void characters(char[] ch, int start, int length) {
	    // store the body, in stages if necessary
	    currentText.append(ch, start, length);
	    string = new String(ch, start, length) ;
	}

	public void endElement(String ns, String ln, String qn)
	    throws SAXException {
	    if (authenticationSuccess) {
		//*** These codes are for backward compatibilty of Yale CAS
		if (qn.equals(USER))
		    user = currentText.toString().trim();
		if (qn.equals(USER_NU))
		    user = currentText.toString().trim();
		if (qn.equals(MAIL))
		    mail = currentText.toString().trim();
		if (qn.equals(FULLNAME))
		    fullName = currentText.toString().trim();
		if (qn.equals(PROXY_GRANTING_TICKET))
		    pgtIou = currentText.toString().trim();
	    } else if (authenticationFailure) {
		if (qn.equals(AUTHENTICATION_FAILURE))
		    errorMessage = currentText.toString().trim();
	    }
	    //*** Encode in Map is for CAS^2
	    String tag = AttributeRewrite.rewrite(qn.substring(CAS_TAG.length())) ;
	    try {
		jp.ac.nagoya_u.cas.client.ServiceTicketValidator.this.result.put(tag, string) ;
		jp.ac.nagoya_u.cas.client.ServiceTicketValidator.this.result.put(tag.toLowerCase(), string) ;
	    } catch (Exception e) {
		//
	    }
	}
 
	public void endDocument() throws SAXException {
	    // save values as appropriate
	    if (authenticationSuccess) {
		jp.ac.nagoya_u.cas.client.ServiceTicketValidator.this.user = user;
		jp.ac.nagoya_u.cas.client.ServiceTicketValidator.this.mail = mail;
		jp.ac.nagoya_u.cas.client.ServiceTicketValidator.this.fullName = fullName;
		jp.ac.nagoya_u.cas.client.ServiceTicketValidator.this.pgtIou = pgtIou;
		jp.ac.nagoya_u.cas.client.ServiceTicketValidator.this.successfulAuthentication = true;
	    } else if (authenticationFailure) {
		jp.ac.nagoya_u.cas.client.ServiceTicketValidator.this.errorMessage = errorMessage;
		jp.ac.nagoya_u.cas.client.ServiceTicketValidator.this.errorCode = errorCode;
		jp.ac.nagoya_u.cas.client.ServiceTicketValidator.this.successfulAuthentication = false;
	    } else
		throw new SAXException("no indication of success or failure from CAS");
	}
    }

    //*********************************************************************
    // Utility methods

    /**
     * Clears internally manufactured state.
     */
    protected void clear() {
	user = pgtIou = errorMessage = null;
	attemptedAuthentication = false;
	successfulAuthentication = false;
    }

    /**
     * Is this ServiceTicketValidator configured
     * to pass renew=true on the ticket validation request?
     * @return true if renew=true on validation reqeust, false otherwise.
     */
    protected boolean isRenew() {
	return renew;
    }

    public String toString(){
	StringBuffer sb = new StringBuffer();
	sb.append("[");
	sb.append(ServiceTicketValidator.class.getName());
	if (casValidateUrl != null){
		sb.append(" casValidateUrl=[");
		sb.append(casValidateUrl);
		sb.append("]");
	}

	if (proxyCallbackUrl != null){
		sb.append(" proxyCallbackUrl=[");
		sb.append(proxyCallbackUrl);
		sb.append("]");
	}
	if (st != null){
		sb.append(" ticket=[");
		sb.append(st);
		sb.append("]");
	}
	if (service != null){
		sb.append(" service=[");
		sb.append(service);
		sb.append("]");
	}
	if (pgtIou != null){
		sb.append(" pgtIou=[");
		sb.append(pgtIou);
		sb.append("]");
	}
	if (user != null){
		sb.append(" user=[");
		sb.append(user);
		sb.append("]");
	}
	if (errorCode != null){
		sb.append(" errorCode=[");
		sb.append(errorCode);
		sb.append("]");
	}
	if (errorMessage != null){
		sb.append(" errorMessage=[");
		sb.append(errorMessage);
		sb.append("]");
	}
	sb.append(" renew=");
	sb.append(renew);
	if (entireResponse != null){
		sb.append(" entireResponse=[");
		sb.append(entireResponse);
		sb.append("]");
	}
	sb.append("]");
	return sb.toString();
    }

}
